#ifndef IMAGE_H
#define IMAGE_H
#include "main.h"

typedef struct point_t point_t;
struct point_t
{
	int x;
	int y;
};

#define POINT(x,y) ((point_t){x,y})

typedef struct image_t image_t;


typedef struct pixel_t pixel_t;
typedef struct pixel_t color_t;
struct pixel_t
{
	uint8_t r;
	uint8_t g;
	uint8_t b;
};

#define PIXEL_RGB(r,g,b) ((pixel_t){r,g,b})
#define COLOR_RGB(r,g,b) ((pixel_t){r,g,b})

#define PIXEL(x) ((pixel_t){(x>>16)&0xff,(x>>8)&0xff,(x)&0xff})
#define COLOR(x) ((pixel_t){(x>>16)&0xff,(x>>8)&0xff,(x)&0xff})

pixel_t* pixel_set(pixel_t* p, uint8_t r, uint8_t g, uint8_t b);

struct image_t
{
	int 			w;
	int 			h;
	uint8_t 	isallocated;
	uint8_t 	isstatic;
	pixel_t*	mat;
};

image_t* image_set	(image_t* p, int w, int h, pixel_t* data);
image_t* image_copy	(image_t* p, int w, int h, pixel_t* data);
image_t* image_empty(image_t* p, int w, int h);
image_t* image_clear(image_t* p);
void		 image_free	(image_t* p);
image_t* image_mem_copy(image_t* out, const image_t* in);

#define IMG_GET(img, i, j)				((img)->mat[(img)->w*(j)+(i)])
#define IMG_GET_R(img, i, j) 			(((img)->mat[(img)->w*(j)+(i)]).r)
#define IMG_GET_G(img, i, j) 			(((img)->mat[(img)->w*(j)+(i)]).g)
#define IMG_GET_B(img, i, j) 			(((img)->mat[(img)->w*(j)+(i)]).b)
#define IMG_GET_M(img, i, j)			(IMG_GET_R(img,i,j)+ IMG_GET_G(img,i,j)+ IMG_GET_B(img,i,j))
#define IMG_GET_Y(img, i, j)			(0.299*IMG_GET_R(img,i,j)+ 0.587*IMG_GET_G(img,i,j)+ 0.114 *IMG_GET_B(img,i,j))

#define IMG_GETi(img, i) 					((img)->mat[i])
#define IMG_GET_Ri(img, i) 				(((img)->mat[i]).r)
#define IMG_GET_Gi(img, i) 				(((img)->mat[i]).g)
#define IMG_GET_Bi(img, i) 				(((img)->mat[i]).b)
#define IMG_GET_Mi(img, i) 				(IMG_GET_Ri(img,i)+ IMG_GET_Gi(img,i)+ IMG_GET_Bi(img,i))
#define IMG_GET_Yi(img, i) 				(0.299*IMG_GET_Ri(img,i)+ 0.587*IMG_GET_Gi(img,i)+ 0.114 *IMG_GET_Bi(img,i))


#define IMG_SET(img, i, j, pix) 	((img)->mat[(img)->w*(j)+(i)]=(pix))
#define IMG_SET_M(img, i, j, pix) IMG_SET_R(img,i,j,pix)+ IMG_SET_G(img,i,j,pix)+ IMG_SET_B(img,i,j,pix)
#define IMG_SET_R(img, i, j, pix) ((img)->mat[(img)->w*(j)+(i)].r=(pix))
#define IMG_SET_G(img, i, j, pix) ((img)->mat[(img)->w*(j)+(i)].g=(pix))
#define IMG_SET_B(img, i, j, pix) ((img)->mat[(img)->w*(j)+(i)].b=(pix))

#define IMG_SETi(img, i, pix) 		(img)->mat[i]=(pix)
#define IMG_SET_Mi(img, i, pix) 	{IMG_SET_Ri(img,i,pix); IMG_SET_Gi(img,i,pix); IMG_SET_Bi(img,i,pix);}
#define IMG_SET_Ri(img, i, pix) 	(img)->mat[i].r=(pix)
#define IMG_SET_Gi(img, i, pix) 	(img)->mat[i].g=(pix)
#define IMG_SET_Bi(img, i, pix) 	(img)->mat[i].b=(pix)

#endif
