/* V4L2 video picture grabber
   Copyright (C) 2009 Mauro Carvalho Chehab <mchehab@infradead.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 */
#include "main.h"
#include "common.h"
#include "image.h"
#include "process.h"


image_t tmpimage;


int sign(int w, int h, uint8_t* s, const uint8_t* a)
{
  int i;
  int max=0;
  int histogramme[257]={0};
  histogramme[256]=1000;
  for(i=0; i<w*h; i++)
  {
    float aa = 0.299*a[i*3]+ 0.587*a[i*3+1]+ 0.114 *a[i*3+2];
    histogramme[(int)aa]++;
  }
  memset(s, 0, w*h*3);
  for(i=0; i<256; i++) 
    if( histogramme[i]>max) max=histogramme[i];
  
  for(i=0; i<257 ;i++)
  {
    histogramme[i]=(histogramme[i]*(h+2))/max;
    s[((h-1)*w+i*2)*3]=0xFF;
    s[((h-1)*w+i*2+1)*3]=0xFF;
    s[((h-1-histogramme[i])*w+i*2)*3]=s[((h-1-histogramme[i])*w+i*2)*3+1]=s[((h-1-histogramme[i])*w+i*2)*3+2]=0xFF;
    s[((h-histogramme[i])*w+i*2)*3]=s[((h-histogramme[i])*w+i*2)*3+1]=s[((h-histogramme[i])*w+i*2)*3+2]=0xFF;
  }
  return 0;
}

int hsm[256]={0};
int sign_moy(int w, int h, uint8_t* s, const uint8_t* in, const uint8_t* a)
{
  int i;
  int max=0;
  int histogramme[257]={0};
  int n=1,x=256;
  double fault=0.0, res;
  histogramme[256]=1000;
  for(i=0; i<w*h; i++)
  {
    float aa = 0.299*a[i*3]+ 0.587*a[i*3+1]+ 0.114 *a[i*3+2];
    histogramme[(int)aa]++;
  }
  memcpy(s, in, w*h*3);
  
  if(1)for(i=n/2;i<256-n/2; i++)
  {
      int moy=0;
      int j;
      for(j=-n/2; j<n/2+1; j++)
        moy+=histogramme[i+j];
      histogramme[i]=moy/n;
  }
  
  for(i=0; i<256; i++) 
    if( histogramme[i]>max) max=histogramme[i];
  
  for(i=0; i<257 ;i++)
  {
    histogramme[i]=(histogramme[i]*(h+2))/max;
    s[((h-1)*w+i*2)*3]=0xFF;
    s[((h-1)*w+i*2+1)*3]=0xFF;
    s[((h-1-histogramme[i])*w+i*2)*3]=s[((h-1-histogramme[i])*w+i*2)*3+1]=s[((h-1-histogramme[i])*w+i*2)*3+2]=0xFF;
    s[((h-2-histogramme[i])*w+i*2)*3]=s[((h-2-histogramme[i])*w+i*2)*3+1]=s[((h-2-histogramme[i])*w+i*2)*3+2]=0xff;
  }
  for(i=0; i<256; i++)
  {
    if(hsm[i] && histogramme[i]) fault+=(histogramme[i]>hsm[i])?(histogramme[i]-hsm[i])/hsm[i]:(hsm[i]-histogramme[i])/histogramme[i];
    else x--;
  }
  res=(100-fault/x);
  printf("Success=%lf %% => ", res);
  if(res>99.9) printf(" OK \n"); else printf(" FAILED\n");
  memcpy(hsm, histogramme, 256);
  return 0;
}

int cb2(sdl_t* sdl, void* data)
{
  vd_t* v = (vd_t*) data;
 /* const int co[25]={1,1,1,1,1,1,1,
                    1,1,1,1,1,1,1,
                    1,1,1,1,1,1,1,
                    1,1,1,1,1,1,1,
                    1,1,1,1,1,1,1,
                    1,1,1,1,1,1,1,
                    1,1,1,1,1,1,1};*/
  vd_shot(v);
  uint8_t tmp[WIDTH*HEIGHT*3];
  /*diff(sdl->width, sdl->height, tmp, 
            (uint8_t*)v->buffers[vd_get_relative_buffer_index(v,0)].start, 
            (uint8_t*)v->buffers[vd_get_relative_buffer_index(v,-1)].start);
*/
            
  sign_moy(sdl->width, sdl->height, (uint8_t*)v->buffertmp, v->buffers[vd_get_relative_buffer_index(v,0)].start, tmp);
  sdl_setimage(sdl,v->buffertmp);

  return 0;
}


int cb(vd_t* v, image_t* out, const image_t* in)
{

  process(out, in);
  
  

  return 0;
}

int main(int argc, char** argv)
{
  vd_t v;
  sdl_t sdl;
  image_empty(&tmpimage, WIDTH, HEIGHT);
  vd_init(&v, DEVICE, WIDTH, HEIGHT, cb);
  sdl_init(&sdl, v.width, v.height);
  sdl_mainloop(&sdl, vd_callback, &v);
  
  sdl_close(&sdl);
  vd_close(&v);
  return 0;
  
}






